"""Super class(es) that is inherited by all API objects."""
from .helper_functions import *
import logging

logging.debug(f"In the {__name__} module.")


class APIClassTemplate(object):
    """
    This class is the base framework for all the objects in the FMC.
    """

    REQUIRED_FOR_POST = ['name']
    REQUIRED_FOR_PUT = ['id']
    REQUIRED_FOR_DELETE = ['id']
    REQUIRED_FOR_GET = ['']
    FILTER_BY_NAME = False
    URL = ''
    URL_SUFFIX = ''
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\-]"""
    FIRST_SUPPORTED_FMC_VERSION = '6.1'
    VALID_JSON_DATA = []
    GLOBAL_VALID_FOR_KWARGS = ['dry_run']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []

    @property
    def show_json(self):
        return self.format_data()

    def __init__(self, fmc, **kwargs):
        logging.debug("In __init__() for APIClassTemplate class.")
        self.VALID_FOR_KWARGS = self.VALID_FOR_KWARGS + self.GLOBAL_VALID_FOR_KWARGS
        self.fmc = fmc
        self.limit = self.fmc.limit
        self.description = 'Created by fmcapi.'
        self.overridable = False
        self.dry_run = False
        self.URL = f'{self.fmc.configuration_url}{self.URL_SUFFIX}'
        if self.fmc.serverVersion < self.FIRST_SUPPORTED_FMC_VERSION:
            logging.warning(f'This API feature was released in version {self.FIRST_SUPPORTED_FMC_VERSION}.  '
                            f'Your FMC version is {self.fmc.serverVersion}.  Upgrade to use this feature.')

    def format_data(self):
        logging.debug("In format_data() for APIClassTemplate class.")
        json_data = {}
        for key_value in self.VALID_JSON_DATA:
            if key_value in self.__dict__:
                json_data[key_value] = self.__dict__[key_value]
        return json_data

    def parse_kwargs(self, **kwargs):
        logging.debug("In parse_kwargs() for APIClassTemplate class.")
        for key_value in self.VALID_FOR_KWARGS:
            if key_value in kwargs:
                self.__dict__[key_value] = kwargs[key_value]
        if 'name' in kwargs:
            self.name = syntax_correcter(kwargs['name'], permitted_syntax=self.VALID_CHARACTERS_FOR_NAME)
            if self.name != kwargs['name']:
                logging.info(f"Adjusting name '{kwargs['name']}' to '{self.name}' due to invalid characters.")

    def valid_for_get(self):
        logging.debug("In valid_for_get() for APIClassTemplate class.")
        if self.REQUIRED_FOR_GET == ['']:
            return True
        for item in self.REQUIRED_FOR_GET:
            if item not in self.__dict__:
                return False
        return True

    def get(self, **kwargs):
        """
        If no self.name or self.id exists then return a full listing of all objects of this type.
        Otherwise set "expanded=true" results for this specific object.
        :return:
        """
        logging.debug("In get() for APIClassTemplate class.")
        self.parse_kwargs(**kwargs)
        if self.fmc.serverVersion < self.FIRST_SUPPORTED_FMC_VERSION:
            logging.error(f'Your FMC version, {self.fmc.serverVersion} does not support GET of this feature.')
            return {'items': []}
        if self.valid_for_get():
            if 'id' in self.__dict__:
                url = f'{self.URL}/{self.id}'
                if self.dry_run:
                    logging.info('Dry Run enabled.  Not actually sending to FMC.  Here is what would have been sent:')
                    logging.info('\tMethod = GET')
                    logging.info(f'\tURL = {self.URL}')
                    return False
                response = self.fmc.send_to_api(method='get', url=url)
                self.parse_kwargs(**response)
                if 'name' in self.__dict__:
                    logging.info(f'GET success. Object with name: "{self.name}" and id: "{self.id}" fetched from FMC.')
                else:
                    logging.info(f'GET success. Object with id: "{self.id}" fetched from FMC.')
            elif 'name' in self.__dict__:
                if self.FILTER_BY_NAME:
                    url = f'{self.URL}?name={self.name}&expanded=true'
                else:
                    url = f'{self.URL}?expanded=true'
                    if 'limit' in self.__dict__:
                        url = f'{url}&limit={self.limit}'
                    if 'offset' in self.__dict__:
                        url = f'{url}&offset={self.offset}'
                response = self.fmc.send_to_api(method='get', url=url)
                if 'items' not in response:
                    response['items'] = []
                for item in response['items']:
                    if 'name' in item:
                        if item['name'] == self.name:
                            self.id = item['id']
                            self.parse_kwargs(**item)
                            logging.info(f'GET success. Object with name: "{self.name}" and id: "{self.id}" '
                                         f'fetched from FMC.')
                            return item
                    else:
                        logging.warning(f'No "name" attribute associated with this item to check against {self.name}.')
                if 'id' not in self.__dict__:
                    logging.warning(f"\tGET query for {self.name} is not found.\n\t\tResponse: {json.dumps(response)}")
            else:
                logging.info("GET query for object with no name or id set.  "
                             "Returning full list of these object types instead.")
                url = f'{self.URL}?expanded=true&limit={self.limit}'
                if self.dry_run:
                    logging.info('Dry Run enabled.  Not actually sending to FMC.  Here is what would have been sent:')
                    logging.info('\tMethod = GET')
                    logging.info(f'\tURL = {self.URL}')
                    return False
                response = self.fmc.send_to_api(method='get', url=url)
            if 'items' not in response:
                response['items'] = []
            return response
        else:
            logging.warning("get() method failed due to failure to pass valid_for_get() test.")
            return False

    def valid_for_post(self):
        logging.debug("In valid_for_post() for APIClassTemplate class.")
        for item in self.REQUIRED_FOR_POST:
            if item not in self.__dict__:
                return False
        return True

    def post(self, **kwargs):
        logging.debug("In post() for APIClassTemplate class.")
        if self.fmc.serverVersion < self.FIRST_SUPPORTED_FMC_VERSION:
            logging.error(f'Your FMC version, {self.fmc.serverVersion} does not support POST of this feature.')
            return False
        if 'id' in self.__dict__:
            logging.info("ID value exists for this object.  Redirecting to put() method.")
            self.put()
        else:
            if self.valid_for_post():
                if self.dry_run:
                    logging.info('Dry Run enabled.  Not actually sending to FMC.  Here is what would have been sent:')
                    logging.info('\tMethod = POST')
                    logging.info(f'\tURL = {self.URL}')
                    logging.info(f'\tJSON = {self.format_data()}')
                    return False
                response = self.fmc.send_to_api(method='post', url=self.URL, json_data=self.format_data())
                if response:
                    self.parse_kwargs(**response)
                    if 'name' in self.__dict__ and 'id' in self.__dict__:
                        logging.info(f'POST success. Object with name: "{self.name}" and id: "{id}" created in FMC.')
                    else:
                        logging.info('POST success but no "id" or "name" values in API response.')
                else:
                    logging.warning('POST failure.  No data in API response.')
                return response
            else:
                logging.warning("post() method failed due to failure to pass valid_for_post() test.")
                return False

    def valid_for_put(self):
        logging.debug("In valid_for_put() for APIClassTemplate class.")
        for item in self.REQUIRED_FOR_PUT:
            if item not in self.__dict__:
                return False
        return True

    def put(self, **kwargs):
        logging.debug("In put() for APIClassTemplate class.")
        self.parse_kwargs(**kwargs)
        if self.fmc.serverVersion < self.FIRST_SUPPORTED_FMC_VERSION:
            logging.error(f'Your FMC version, {self.fmc.serverVersion} does not support PUT of this feature.')
            return False
        if self.valid_for_put():
            url = f'{self.URL}/{self.id}'
            if self.dry_run:
                logging.info('Dry Run enabled.  Not actually sending to FMC.  Here is what would have been sent:')
                logging.info('\tMethod = PUT')
                logging.info(f'\tURL = {self.URL}')
                logging.info(f'\tJSON = {self.format_data()}')
                return False
            response = self.fmc.send_to_api(method='put', url=url, json_data=self.format_data())
            self.parse_kwargs(**response)
            if 'name' in self.__dict__:
                logging.info(f'PUT success. Object with name: "{self.name}" and id: "{self.id}" updated in FMC.')
            else:
                logging.info(f'PUT success. Object with id: "{self.id}" updated in FMC.')
            return response
        else:
            logging.warning("put() method failed due to failure to pass valid_for_put() test.")
            return False

    def valid_for_delete(self):
        logging.debug("In valid_for_delete() for APIClassTemplate class.")
        for item in self.REQUIRED_FOR_DELETE:
            if item not in self.__dict__:
                return False
        return True

    def delete(self, **kwargs):
        logging.debug("In delete() for APIClassTemplate class.")
        self.parse_kwargs(**kwargs)
        if self.fmc.serverVersion < self.FIRST_SUPPORTED_FMC_VERSION:
            logging.error(f'Your FMC version, {self.fmc.serverVersion} does not support DELETE of this feature.')
            return False
        if self.valid_for_delete():
            url = f'{self.URL}/{self.id}'
            if self.dry_run:
                logging.info('Dry Run enabled.  Not actually sending to FMC.  Here is what would have been sent:')
                logging.info('\tMethod = DELETE')
                logging.info(f'\tURL = {self.URL}')
                logging.info(f'\tJSON = {self.format_data()}')
                return False
            response = self.fmc.send_to_api(method='delete', url=url, json_data=self.format_data())
            if not response:
                return None
            self.parse_kwargs(**response)
            if 'name' in self.name:
                logging.info(f'DELETE success. Object with name: "{self.name}" and id: "{self.id}" deleted in FMC.')
            else:
                logging.info(f'DELETE success. Object id: "{self.id}" deleted in FMC.')
            return response
        else:
            logging.warning("delete() method failed due to failure to pass valid_for_delete() test.")
            return False



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class FTDNatPolicies(APIClassTemplate):
    """
    The FTDNatPolicies Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/policy/ftdnatpolicies'
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for FTDNatPolicies class.")
        self.parse_kwargs(**kwargs)
        self.type = "FTDNatPolicy"


class FTDNatPolicy(FTDNatPolicies):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: FTDNatPolicy() should be called via FTDNatPolicies().")
        super().__init__(fmc, **kwargs)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class SecurityZones(APIClassTemplate):
    """
    The SecurityZones Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'description', 'interfaceMode', 'interfaces']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/securityzones'
    REQUIRED_FOR_POST = ['name', 'interfaceMode']
    FILTER_BY_NAME = True

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for SecurityZones class.")
        self.parse_kwargs(**kwargs)

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for SecurityZones class.")
        if 'interfaceMode' in kwargs:
            self.interfaceMode = kwargs['interfaceMode']
        else:
            self.interfaceMode = 'ROUTED'


class SecurityZone(SecurityZones):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: SecurityZone() should be called via SecurityZones().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi


def test__application(fmc):
    logging.info('Testing Application class.')
    
    obj1 = fmcapi.Applications(fmc=fmc)
    logging.info('All Application -- >')
    result = obj1.get(limit=1000)
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")

    del obj1
    obj1 = fmcapi.Applications(fmc=fmc, name='WD softwares Download/Update')
    logging.info('One Application -- >')
    logging.info(obj1.get(limit=1000))

    logging.info('Testing Application class done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging


class DNSServerGroups(APIClassTemplate):
    """
    The DNSServerGroups Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'retries', 'timeout', 'dnsservers', 'defaultdomain']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/dnsservergroups'
    REQUIRED_FOR_POST = ['name', 'timeout']
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for DNSServerGroups class.")
        self.parse_kwargs(**kwargs)
        self.type = 'DNSServerGroupObject'

    def servers(self, action, name_servers):
        logging.debug("In servers() for DNSServerGroups class.")
        if action == 'add':
            for name_server in name_servers:
                if 'dnsservers' in self.__dict__:
                    self.dnsservers.append({"name-server":name_server})
                else:
                    self.dnsservers = [{"name-server":name_server}]
                logging.info(f'Name-server "{name_server}" added to this DNSServerGroups object.')
        elif action == 'remove':
            if 'dnsservers' in self.__dict__:
                for name_server in name_servers:
                    self.dnsservers = list(filter(lambda i: i['name-server'] != name_server, self.dnsservers))
            else:
                logging.warning('DNSServerGroups has no members.  Cannot remove name-server.')
        elif action == 'clear':
            if 'dnsservers' in self.__dict__:
                del self.dnsservers
                logging.info('All name-servers removed from this DNSServerGroups object.')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class Countries(APIClassTemplate):
    """
    The Countries Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'iso2', 'iso3']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/countries'
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for Countries class.")
        self.parse_kwargs(**kwargs)
        self.type = 'Country'

    def post(self):
        logging.info('POST method for API for Countries not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for Countries not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for Countries not supported.')
        pass


class Country(Countries):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: Country() should be called via Countries().")
        super().__init__(fmc, **kwargs)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging


class FQDNS(APIClassTemplate):
    """
    The FQDNS Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'overridableTargetId', 'value', 'dnsResolution', 'overrides',
                       'overridable',
                       ]
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/fqdns'
    VALID_FOR_DNS_RESOLUTION = ['IPV4_ONLY', 'IPV6_ONLY', 'IPV4_AND_IPV6']
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""
    FIRST_SUPPORTED_FMC_VERSION = '6.3.0'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for FQDNS class.")
        self.parse_kwargs(**kwargs)
        self.type = 'FQDN'
        if self.fmc.serverVersion < self.FIRST_SUPPORTED_FMC_VERSION:
            logging.warning(f'The FQDNS API feature was released in version {self.FIRST_SUPPORTED_FMC_VERSION}.  '
                            f'Your FMC version is {self.fmc.serverVersion}.  Upgrade to use this feature.')

    def format_data(self):
        json_data = super().format_data()
        logging.debug("In format_data() for FQDNS class.")
        if 'dnsResolution' in self.__dict__:
            if self.dnsResolution in self.VALID_FOR_DNS_RESOLUTION:
                json_data['dnsResolution'] = self.dnsResolution
            else:
                logging.warning(f'dnsResolution {self.dnsResolution} not a valid type.')
        return json_data

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for FQDNS class.")
        if 'dnsResolution' in kwargs:
            if kwargs['dnsResolution'] in self.VALID_FOR_DNS_RESOLUTION:
                self.dnsResolution = kwargs['dnsResolution']
            else:
                logging.warning(f"dnsResolution {kwargs['dnsResolution']} not a valid type.")



import logging
import fmcapi
import time
from unit_tests import wait_for_task


def test__upgrades(fmc):
    logging.info(
        'Test UpgradePackages/ApplicableDevices/Upgrades with Task.'
        '  This will copy the listed upgrade file to registered devices')

    package_name = 'Cisco_FTD_Patch-6.3.0.3-77.sh.REL.tar'
    device_list = []

    logging.info('All UpgradePackages -- >')
    package1 = fmcapi.UpgradePackages(fmc=fmc)
    result = package1.get()
    logging.info(result)
    del package1

    package1 = fmcapi.UpgradePackages(fmc=fmc, name=package_name)
    logging.info('One UpgradePackage -- >')
    logging.info(package1.get())

    applicable1 = fmcapi.ListApplicableDevices(fmc=fmc)
    applicable1.upgrade_package(package_name=package_name)
    time.sleep(1)
    result = applicable1.get()
    devices = result.get('items', [])

    for device in devices:
        device_list.append(device['name'])

    upgrades1 = fmcapi.Upgrades(fmc=fmc)
    upgrades1.devices(devices=device_list)
    upgrades1.upgrade_package(package_name=package_name)
    upgrades1.pushUpgradeFileOnly = True

    response = upgrades1.post()
    logging.info(response)
    wait_for_task(fmc=fmc, task=response["metadata"]["task"], wait_time=60)

    logging.info('Test UpgradePackages/ApplicableDevices/Upgrades Complete')



import logging
import fmcapi


def test__fmc_version(fmc):
    logging.info('Testing fmc.version() method.  Getting version information information from FMC.')

    version_info = fmc.serverversion()
    logging.info('fmc.version() -- >')
    logging.info(version_info)

    logging.info('Testing fmc.version() done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.device_services.physicalinterfaces import PhysicalInterfaces
import logging
import warnings


class InterfaceGroups(APIClassTemplate):
    """
    The InterfaceGroups Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'description', 'interfaceMode', 'interfaces']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/interfacegroups'
    REQUIRED_FOR_POST = ['name', 'interfaceMode']
    REQUIRED_FOR_PUT = ['id']
    FILTER_BY_NAME = True

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for InterfaceGroups class.")
        self.parse_kwargs(**kwargs)
        self.type = 'InterfaceGroup'

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for InterfaceGroups class.")
        if 'interfaceMode' in kwargs:
            self.interfaceMode = kwargs['interfaceMode']
        else:
            self.interfaceMode = 'ROUTED'

    def p_interface(self, device_name="", action="add", names=[]):
        logging.debug("In interfaces() for InterfaceGroups class.")
        if action == 'add':
            intfs = []
            for name in names:
                intf = PhysicalInterfaces(fmc=self.fmc)
                intf.get(name=name, device_name=device_name)
                if 'id' in intf.__dict__ and 'ifname' in intf.__dict__:
                    intfs.append({'name': intf.name, 'id': intf.id, 'type': intf.type})
                elif 'id' in intf.__dict__:
                    logging.warning(f'PhysicalInterface, "{name}", found without logical ifname.  '
                                    f'Cannot add to InterfaceGroups.')
                else:
                    logging.warning(f'PhysicalInterface, "{name}", not found.  Cannot add to InterfaceGroups.')
            if len(intfs) != 0:
                # Make sure we found at least one intf
                self.interfaces = intfs
            else:
                logging.warning(f'No valid PhysicalInterface found: "{names}".  Cannot remove from InterfaceGroups.')
        elif action == 'remove':
            if 'interfaces' in self.__dict__:
                intfs = []
                for interface in self.interfaces:
                    if interface["name"] not in names:
                        intfs.append(interface)
                    else:
                        logging.info(f"""Removed "{interface['name']}" from InterfaceGroups.""")
                self.interfaces = intfs
            else:
                logging.warning("This InterfaceObject has no interfaces.  Nothing to remove.")
        elif action == 'clear-all':
            if 'interfaces' in self.__dict__:
                del self.interfaces
                logging.info('All PhysicalInterfaces removed from this InterfaceGroups.')


class InterfaceGroup(InterfaceGroups):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: InterfaceGroup() should be called via InterfaceGroups().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi


def test__extended_acls(fmc):
    logging.info('Testing ExtendedAccessList class. Requires a configured ExtendedAccessList')

    obj1 = fmcapi.ExtendedAccessList(fmc=fmc)
    logging.info('All ExtendedAccessList -- >')
    result = obj1.get()
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")
    del obj1

    logging.info('Testing ExtendedAccessList class done.\n')



import logging
from .listapplicabledevices import ListApplicableDevices
from .listapplicabledevices import ApplicableDevices
from .upgradepackages import UpgradePackages
from .upgradepackages import UpgradePackage
from .upgradepackage import Upgrades

logging.debug("In the update_packages __init__.py file.")

__all__ = [
    'ListApplicableDevices',
    'ApplicableDevices',
    'UpgradePackages',
    'UpgradePackage',
    'Upgrades',
]


import logging
from .ftddevicehapairs import FTDDeviceHAPairs
from .ftddevicehapairs import DeviceHAPairs
from .failoverinterfacemacaddressconfigs import FailoverInterfaceMACAddressConfigs
from .failoverinterfacemacaddressconfigs import DeviceHAFailoverMAC
from .monitoredinterfaces import DeviceHAMonitoredInterfaces
from .monitoredinterfaces import MonitoredInterfaces

logging.debug("In the device_ha_pair_services __init__.py file.")

__all__ = [
    'FTDDeviceHAPairs',
    'DeviceHAPairs',
    'FailoverInterfaceMACAddressConfigs',
    'DeviceHAFailoverMAC',
    'DeviceHAMonitoredInterfaces',
    'MonitoredInterfaces'
]



import logging
import fmcapi
import time


def test__dns_servers_group(fmc):
    logging.info('Test DNSServerGroups.  Post, get, put, delete DNSServerGroups Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    server_list = ["192.0.2.1", "192.0.2.2"]
    obj1 = fmcapi.DNSServerGroups(fmc=fmc)
    obj1.name = "_dns1" + namer
    obj1.timeout = "3"
    obj1.defaultdomain = "cisco.com"
    obj1.post()

    obj1.get()
    obj1.servers(action='add', name_servers=server_list)
    obj1.put()

    obj1.delete()

    logging.info('Testing DNSServerGroups class done.\n')



import logging
import fmcapi


def test__application_type(fmc):
    logging.info('Testing ApplicationType class.')
    
    obj1 = fmcapi.ApplicationTypes(fmc=fmc)
    logging.info('All ApplicationType -- >')
    result = obj1.get()
    logging.info(result)
    logging.info(f"Total items: {len(result['items'])}")

    del obj1
    obj1 = fmcapi.ApplicationTypes(fmc=fmc, name='Server')
    logging.info('One ApplicationType -- >')
    logging.info(obj1.get())

    logging.info('Testing ApplicationType class done.\n')



"""
Misc methods/functions that are used by the fmcapi package's modules.
"""

import re
import ipaddress
import json
import logging

logging.debug(f"In the {__name__} module.")


def syntax_correcter(value, permitted_syntax="""[.\w\d_\-]""", replacer='_'):
    """
    Check 'value' for invalid characters (identified by 'permitted_syntax') and replace them with 'replacer'.
    :param value:  String to be checked.
    :param permitted_syntax: (optional) regex of allowed characters.
    :param replacer: (optional) character used to replace invalid characters.
    :return: Modified string with "updated" characters.
    """
    logging.debug("In syntax_correcter() helper_function.")
    new_value = ''
    for char in range(0, len(value)):
        if not re.match(permitted_syntax, value[char]):
            new_value += replacer
        else:
            new_value += value[char]
    return new_value


def get_networkaddress_type(value):
    """
    Check to see whether 'value' is a host, range, or network.
    :param value: 
    :return: 'host'/'network'/'range'
    """
    logging.debug("In get_networkaddress_type() helper_function.")
    if '/' in value:
        ip, bitmask = value.split('/')
        if ip == '32' or bitmask == '128':
            return 'host'
        else:
            return 'network'
    else:
        if '-' in value:
            return 'range'
        else:
            return 'host'


def is_ip(ip):
    """
    Checks to see whether the provided string is an IP address.
    :param ip: String
    :return: True/False
    """
    logging.debug("In is_ip() helper_function.")
    try:
        ipaddress.ip_address(ip)
    except ValueError as err:
        logging.error(err)
        return False
    return True


def is_ip_network(ip):
    """
    Checks to see whether the provided string is a valid network address.  That is, it checks to see if the
     provided IP/SM is the "network address" of the subnet.
    :param ip: String
    :return: True/False
    """
    logging.debug("In is_ip_network() helper_function.")
    try:
        ipaddress.ip_network(ip)
    except ValueError as err:
        logging.error(err)
        return False
    return True


def validate_ip_bitmask_range(value='', value_type=''):
    """
    We need to check the provided IP address (or range of addresses) and make sure the IPs are valid.
    :param value: IP, IP/Bitmask, or IP Range
    :param value_type: 'host'/'network'/'range'
    :return: dict {value=value_fixed, valid=boolean}
    """
    logging.debug("In validate_ip_bitmask_range() helper_function.")
    return_dict = {'value': value, 'valid': False}
    if value_type == 'range':
        for ip in value.split('-'):
            if is_ip(ip):
                return_dict['valid'] = True
    elif value_type == 'host' or value_type == 'network':
        if is_ip_network(value):
            return_dict['valid'] = True
    return return_dict['valid']


def mocked_requests_get(**kwargs):
    """
    Use to "mock up" a response from using the "requests" library to avoid actually using the "requests" library.
    :param kwargs: 
    :return: 
    """
    logging.debug("In mocked_requests_get() helper_function.")

    class MockResponse:
        def __init__(self, **kwargs):
            logging.debug("In MockResponse __init__ method.")
            self.text = json.dumps(kwargs['text'])
            self.status_code = kwargs['status_code']

        def close(self):
            logging.debug("In MockResponse close method.")
            return True
    return MockResponse(**kwargs)


def validate_vlans(start_vlan, end_vlan=''):
        logging.debug("In validate_vlans() helper_function.")
        if end_vlan == '':
            end_vlan = start_vlan
        if int(end_vlan) < int(start_vlan):
            start_vlan, end_vlan = end_vlan, start_vlan
        if int(start_vlan) > 0 and int(start_vlan) < 4095 and int(end_vlan) > 0 and int(end_vlan) < 4095:
            return start_vlan, end_vlan
        else:
            return 1, 4094



import logging
import fmcapi
import time


def test__ikev2(fmc):
    logging.info(
        'Test IKEv2Policies and IKEv2IpsecProposals.'
        '  Post, get, put, delete IKEv2Policies and IKEv2IpsecProposals Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    encryption_list = ['AES', 'AES-192', 'AES-256', 'NULL']
    integrity_list1 = ['NULL', 'SHA-1', 'SHA-256', 'SHA-384', 'SHA-512']
    ipsec_integrity_list1 = ['NULL', 'SHA', 'SHA-256', 'SHA-384', 'SHA-512']
    # 'NULL' is invalid for prf_integrity.  Should generate a warning log and ignore that type.
    prf_integrity_list1 = ['NULL', 'SHA', 'SHA-256', 'SHA-384', 'SHA-512']

    ipsec1 = fmcapi.IKEv2IpsecProposals(fmc=fmc)
    ipsec1.name = "_ipsec" + namer
    ipsec1.encryption(action='add', algorithms=encryption_list)
    ipsec1.hash(action='add', algorithms=integrity_list1)
    ipsec1.post()

    ipsec1.get()
    # Try to add a duplicate
    ipsec1.encryption(action='add', algorithms=['AES-192'])
    ipsec1.hash(action='add', algorithms=['SHA-1'])

    ipsec1.encryption(action='remove', algorithms=['NULL'])
    ipsec1.hash(action='remove', algorithms=['NULL'])
    ipsec1.put()

    # None of the algorithms can contain an empty list
    ipsec1.get()
    ipsec1.encryption(action='clear')
    ipsec1.hash(action='clear')
    ipsec1.encryption(action='add', algorithms=['AES-192'])
    ipsec1.hash(action='add', algorithms=['SHA-1'])
    ipsec1.put()

    pol1 = fmcapi.IKEv2Policies(fmc=fmc)
    pol1.name = "_pol" + namer
    pol1.priority = "10"
    pol1.diffieHellmanGroups = ["2", "5"]
    pol1.encryption(action='add', algorithms=encryption_list)
    pol1.hash(action='add', algorithms=ipsec_integrity_list1)
    pol1.prf_hash(action='add', algorithms=prf_integrity_list1)
    pol1.lifetimeInSeconds = "3600"
    pol1.post()

    pol1.get()
    # Try to add a duplicate
    pol1.encryption(action='add', algorithms=['AES-192'])
    pol1.hash(action='add', algorithms=['NULL'])
    pol1.prf_hash(action='add', algorithms=['SHA'])

    pol1.encryption(action='remove', algorithms=['NULL'])
    pol1.hash(action='remove', algorithms=['NULL'])
    pol1.prf_hash(action='remove', algorithms=['SHA'])
    pol1.put()

    # None of the algorithms can contain an empty list
    pol1.get()
    pol1.encryption(action='clear')
    pol1.hash(action='clear')
    pol1.prf_hash(action='clear')
    pol1.encryption(action='add', algorithms=['AES-192'])
    pol1.hash(action='add', algorithms=['NULL'])
    pol1.prf_hash(action='add', algorithms=['SHA'])
    pol1.put()

    ipsec1.delete()
    pol1.delete()

    logging.info('Test IKEv2Policies and IKEv2IpsecProposals classes done.\n')



import logging
import fmcapi
import time


def test__icmpv4(fmc):
    logging.info('Test ICMPv4Object.  Post, get, put, delete ICMPv4Object Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    obj1 = fmcapi.ICMPv4Objects(fmc=fmc)
    obj1.name = "_icmpv4" + namer
    obj1.icmpType = "3"
    obj1.code = "0"
    obj1.post()

    obj1.get()
    obj1.code = "3"
    obj1.put()

    obj1.delete()

    logging.info('ICMPv4Object class done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging


class IKEv1IpsecProposals(APIClassTemplate):
    """
    The IKEv1IpsecProposals Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'espEncryption', 'espHash']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/ikev1ipsecproposals'
    REQUIRED_FOR_POST = ['name', 'espEncryption', 'espHash']
    VALID_FOR_ENCRYPTION = ['DES', '3DES', 'AES-128', 'AES-192', 'AES-256', 'ESP-NULL']
    VALID_FOR_HASH = ['NONE', 'MD5', 'SHA']
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""
    FIRST_SUPPORTED_FMC_VERSION = '6.3'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for IKEv1IpsecProposals class.")
        self.parse_kwargs(**kwargs)
        self.type = 'IKEv1IPsecProposal'



import logging
import fmcapi
import time


def test__etherchannel_interfaces(fmc):
    logging.info(
        'Test EtherchannelInterfaces.  get, post, put, delete EtherchannelInterfaces Objects. '
        'Requires registered physical device')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    sz1 = fmcapi.SecurityZones(fmc=fmc)
    sz1.name = "_sz1" + namer
    sz1.post()
    time.sleep(1)
    sz2 = fmcapi.SecurityZones(fmc=fmc)
    sz2.name = "_sz2" + namer
    sz2.post()
    time.sleep(1)

    eth1 = fmcapi.EtherchannelInterfaces(fmc=fmc, device_name="device_name")
    eth1.p_interfaces(p_interfaces=["GigabitEthernet0/3", "GigabitEthernet0/5"], device_name="device_name")
    eth1.enabled = True
    eth1.ifname = "_eth1" + namer
    eth1.etherChannelId = "1"
    eth1.static(ipv4addr="192.0.2.1", ipv4mask=24)
    eth1.sz(name=sz1.name)
    eth1.mode = "NONE"
    eth1.MTU = "1500"
    eth1.lacpMode = "ACTIVE"
    eth1.loadBalancing = "SRC_DST_IP_PORT"
    eth1.post()
    time.sleep(2)

    eth1.get()
    eth1.enabled = False
    eth1.sz(name=sz2.name)
    eth1.put()
    time.sleep(1)

    eth1.get()
    eth1.delete()
    sz1.delete()
    sz2.delete()

    logging.info('Testing EtherchannelInterfaces class done.\n')



"""
This module (fmc.py) is designed to provide a "toolbox" of tools for interacting with the Cisco FMC API.
The "toolbox" is the FMC class and the "tools" are its methods.

Note: There exists a "Quick Start Guide" for the Cisco FMC API too.  Just Google for it as it gets updated with each
 release of code.
"""

import datetime
import requests
import time
import json
from requests.packages.urllib3.exceptions import InsecureRequestWarning
import logging
from logging.handlers import RotatingFileHandler
import warnings

# Disable annoying HTTP warnings
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

""""
The 'requests' package is very chatty on the INFO logging level.  Change its logging threshold sent to logger to 
something greater than INFO (i.e. not INFO or DEBUG) will cause it to not log its INFO and DEBUG messages to the 
default logger.  This reduces the size of our log files.
"""
logging.getLogger("requests").setLevel(logging.WARNING)


# noinspection SpellCheckingInspection
class FMC(object):
    """
The FMC class has a series of methods, lines that start with "def", that are used to interact with the Cisco FMC
via its API.  Each method has its own DOCSTRING (like this triple quoted text here) describing its functionality.
    """
    logging.debug("In the FMC() class.")

    API_CONFIG_VERSION = 'api/fmc_config/v1'
    API_PLATFORM_VERSION = 'api/fmc_platform/v1'
    VERIFY_CERT = False
    MAX_PAGING_REQUESTS = 2000
    TOO_MANY_CONNECTIONS_TIMEOUT = 30
    FMC_MAX_PAYLOAD = 2048000

    def __init__(self,
                 host='192.168.45.45',
                 username='admin',
                 password='Admin123',
                 domain=None,
                 autodeploy=True,
                 file_logging=None,
                 logging_level='INFO',
                 debug=False,
                 limit=1000):
        """
        Instantiate some variables prior to calling the __enter__() method.
        :param host:
        :param username:
        :param password:
        :param autodeploy:
        :param file_logging (str): The filename (and optional path) of the output file if a file logger is required,
        None if no file logger is required
        :param logging_level (str): The desired logging level, INFO by default.
        :param debug (bool): True to enable debug logging, default is False
        :param limit (int): Sets up max page of data to gather per "page".
        """

        if debug:
            logging_level = 'DEBUG'
        root_logger = logging.getLogger('')
        if logging_level.upper() == 'DEBUG':
            root_logger.setLevel(logging.DEBUG)
        if logging_level.upper() == 'INFO':
            root_logger.setLevel(logging.INFO)
        if logging_level.upper() == 'WARNING':
            root_logger.setLevel(logging.WARNING)
        if logging_level.upper() == 'ERROR':
            root_logger.setLevel(logging.ERROR)
        if logging_level.upper() == 'CRITICAL':
            root_logger.setLevel(logging.CRITICAL)

        if file_logging:
            print(f'Logging is enabled and set to {logging_level}.  Look for file "{file_logging}" for output.')
            formatter = logging.Formatter('%(asctime)s - %(levelname)s:%(filename)s:%(lineno)s - %(message)s',
                                          '%Y/%m/%d-%H:%M:%S')
            file_logger = RotatingFileHandler(file_logging, maxBytes=1024000, backupCount=10, mode='w')
            file_logger.setFormatter(formatter)
            root_logger.addHandler(file_logger)

        logging.debug("In the FMC __init__() class method.")

        self.host = host
        self.username = username
        self.password = password
        self.domain = domain
        self.autodeploy = autodeploy
        self.limit = limit
        self.vdbVersion = None
        self.sruVersion = None
        self.serverVersion = None
        self.geoVersion = None
        self.configuration_url = None
        self.platform_url = None
        self.page_counter = None

    def __enter__(self):
        """
        Get a token from the FMC as well as the Global UUID.  With this information set up the base_url variable.
        :return:
        """
        logging.debug("In the FMC __enter__() class method.")
        self.mytoken = Token(host=self.host,
                             username=self.username,
                             password=self.password,
                             domain=self.domain,
                             verify_cert=self.VERIFY_CERT)
        self.uuid = self.mytoken.uuid
        self.build_urls()
        self.serverversion()
        logging.info(f"This FMC's version is {self.serverVersion}")
        return self

    def __exit__(self, *args):
        """
        If autodeploy == True push changes to FMC upon exit of "with" contract.
        :param args:
        :return:
        """
        logging.debug("In the FMC __exit__() class method.")

        if self.autodeploy:
            self.deploymentrequests()
        else:
            logging.info("Auto deploy changes set to False.  "
                         "Use the Deploy button in FMC to push changes to FTDs.\n\n")

    def build_urls(self):
        """
        The FMC APIs appear to use 2 base URLs, depending on what that API is for.  One for "configuration" and the
        other for FMC "platform" things.
        """
        logging.debug("In the FMC build_urls() class method.")
        logging.info('Building base to URLs.')
        self.configuration_url = f"https://{self.host}/{self.API_CONFIG_VERSION}/domain/{self.uuid}"
        self.platform_url = f"https://{self.host}/{self.API_PLATFORM_VERSION}"

    def send_to_api(self, method='', url='', headers='', json_data=None, more_items=[]):
        """
        Using the "method" type, send a request to the "url" with the "json_data" as the payload.
        :param method:
        :param url:
        :param headers:
        :param json_data:
        :param more_items:
        :return:
        """
        logging.debug("In the FMC send_to_api() class method.")

        if not more_items:
            self.more_items = []
            self.page_counter = 0
        if headers == '':
            # These values for headers works for most API requests.
            headers = {'Content-Type': 'application/json', 'X-auth-access-token': self.mytoken.get_token()}
        status_code = 429
        response = None
        json_response = None
        try:
            while status_code == 429:
                if method == 'get':
                    response = requests.get(url, headers=headers, verify=self.VERIFY_CERT)
                elif method == 'post':
                    response = requests.post(url, json=json_data, headers=headers, verify=self.VERIFY_CERT)
                elif method == 'put':
                    response = requests.put(url, json=json_data, headers=headers, verify=self.VERIFY_CERT)
                elif method == 'delete':
                    response = requests.delete(url, headers=headers, verify=self.VERIFY_CERT)
                else:
                    logging.error("No request method given.  Returning nothing.")
                    return
                status_code = response.status_code
                if status_code == 429:
                    logging.warning(f"Too many connections to the FMC.  Waiting {self.TOO_MANY_CONNECTIONS_TIMEOUT} "
                                    f"seconds and trying again.")
                    time.sleep(self.TOO_MANY_CONNECTIONS_TIMEOUT)
                if status_code == 401:
                    logging.warning("Token has expired. Trying to refresh.")
                    self.mytoken.access_token = self.mytoken.get_token()
                    headers = {'Content-Type': 'application/json', 'X-auth-access-token': self.mytoken.access_token}
                    status_code = 429
                if status_code == 422:
                    logging.warning("Either:\n\t1. Payload too large.  FMC can only handle a payload of "
                                    f"{self.FMC_MAX_PAYLOAD} bytes.\n\t2.The payload contains an unprocessable or "
                                    f"unreadable entity such as a invalid attribut name or incorrect JSON syntax ")
            json_response = json.loads(response.text)
            if status_code > 301 or 'error' in json_response:
                response.raise_for_status()
        except requests.exceptions.HTTPError as err:
            logging.error(f"Error in POST operation --> {str(err)}")
            logging.error(f"json_response -->\t{json_response}")
            if response:
                response.close()
            return None
        if response:
            response.close()
        try:
            if 'next' in json_response['paging'] and self.page_counter <= self.MAX_PAGING_REQUESTS:
                self.more_items += json_response['items']
                logging.info(f"Paging:  Offset:{json_response['paging']['offset']}, "
                             f"Limit:{json_response['paging']['limit']}, "
                             f"Count:{json_response['paging']['count']}, "
                             f"Gathered_Items:{len(self.more_items)}.")
                self.page_counter += 1
                return self.send_to_api(method=method,
                                        url=json_response['paging']['next'][0],
                                        json_data=json_data,
                                        more_items=self.more_items)
            else:
                json_response['items'] += self.more_items
                self.more_items = []
                return json_response
        except KeyError:
            # Used only when the response only has "one page" of results.
            return json_response

    def serverversion(self):
        """
        Get the FMC's version information.  Set instance variables for each version info returned as well as return
        the whole response text.
        :return:
        """
        logging.debug("In the FMC version() class method.")
        logging.info('Collecting version information from FMC.')

        url_suffix = '/info/serverversion'
        url = f'{self.platform_url}{url_suffix}'

        response = self.send_to_api(method='get', url=url)
        if 'items' in response:
            logging.info('Populating vdbVersion, sruVersion, serverVersion, and geoVersion FMC instance variables.')
            self.vdbVersion = response['items'][0]['vdbVersion']
            self.sruVersion = response['items'][0]['sruVersion']
            self.serverVersion = response['items'][0]['serverVersion']
            self.geoVersion = response['items'][0]['geoVersion']
        return response

    def version(self):
        """Dispose of this method after 20210101."""
        warnings.warn("Deprecated: version() should be called via serverversion().")
        self.serverversion()

    def auditrecords(self):
        """
        This API function supports filtering the GET query URL with: username, subsystem, source, starttime, and
        endtime parameters.
        :return: response
        """
        logging.debug('In the auditrecords method of FMC.')
        url_parameters = 'expanded=true'
        url_suffix = '/audit/auditrecords'
        url = f'{self.platform_url}/domain/{self.uuid}{url_suffix}?{url_parameters}'

        response = self.send_to_api(method='get', url=url)
        return response

    def audit(self):
        """Dispose of this method after 20210101."""
        warnings.warn("Deprecated: audit() should be called via auditrecords().")
        self.auditrecords()

    def deployabledevices(self):
        """
        Collect a list of FMC managed devices who's configuration is not up-to-date.
        :return: List of devices needing updates.
        """
        logging.debug("In the FMC deployabledevices() class method.")

        waittime = 15
        logging.info(f"Waiting {waittime} seconds to allow the FMC to update the list of deployable devices.")
        time.sleep(waittime)
        logging.info("Getting a list of deployable devices.")
        url_suffix = "/deployment/deployabledevices?expanded=true"
        url = f'{self.configuration_url}{url_suffix}'
        response = self.send_to_api(method='get', url=url)
        # Now to parse the response list to get the UUIDs of each device.
        if 'items' not in response:
            return
        uuids = []
        for item in response['items']:
            if not item['canBeDeployed']:
                pass
            else:
                uuids.append(item)
        return uuids

    def get_deployable_devices(self):
        """Dispose of this method after 20210101."""
        warnings.warn("Deprecated: get_deployable_devices() should be called via deployabledevices().")
        self.deployabledevices()

    def deploymentrequests(self):
        """
        Iterate through the list of devices needing deployed and submit a request to the FMC to deploy changes to them.
        :return:
        """
        logging.debug("In the deploymentrequests() class method.")

        url_suffix = "/deployment/deploymentrequests"
        url = f'{self.configuration_url}{url_suffix}'
        devices = self.deployabledevices()
        if not devices:
            logging.info("No devices need deployed.\n\n")
            return
        json_data = {
            'type': 'DeploymentRequest',
            'forceDeploy': True,
            'ignoreWarning': True,
            'version': str(int(1000000 * datetime.datetime.utcnow().timestamp())),
            'deviceList': []
        }
        for device in devices:
            logging.info(f"Adding device {device} to deployment queue.")
            json_data['deviceList'].append(device['device']['id'])
            # From the list of deployable devices get the version value that is smallest.
            if int(json_data['version']) > int(device['version']):
                logging.info(f"Updating version to {device['version']}")
                json_data['version'] = device['version']

        logging.info("Deploying changes to devices.")
        response = self.send_to_api(method='post', url=url, json_data=json_data)
        return response['deviceList']

    def deploy_changes(self):
        """Dispose of this method after 20210101."""
        warnings.warn("Deprecated: deploy_changes() should be called via deploymentrequests().")
        self.deploymentrequests()


class Token(object):
    """
    The token is the validation object used with the FMC.

    """
    logging.debug("In the Token class.")

    MAX_REFRESHES = 3
    TOKEN_LIFETIME = 60 * 30
    TOKEN_REFRESH_TIME = int(TOKEN_LIFETIME * .95)  # Refresh token at 95% refresh time.
    API_PLATFORM_VERSION = 'api/fmc_platform/v1'

    def __init__(self, host='192.168.45.45', username='admin', password='Admin123', domain=None, verify_cert=False):
        """
        Initialize variables used in the Token class.
        :param host:
        :param username:
        :param password:
        :param verify_cert:
        """
        logging.debug("In the Token __init__() class method.")

        self.__host = host
        self.__username = username
        self.__password = password
        self.__domain = domain
        self.uuid = None
        self.verify_cert = verify_cert
        self.token_refreshes = 0
        self.access_token = None
        self.refresh_token = None
        self.token_creation_time = None
        self.generate_tokens()

    def generate_tokens(self):
        """
        Create new and refresh expired tokens.
        :return:
        """
        logging.debug("In the Token generate_tokens() class method.")

        if self.token_refreshes <= self.MAX_REFRESHES and self.access_token is not None:
            headers = {'Content-Type': 'application/json', 'X-auth-access-token': self.access_token,
                       'X-auth-refresh-token': self.refresh_token}
            url = f'https://{self.__host}/{self.API_PLATFORM_VERSION}/auth/refreshtoken'
            logging.info(f"Refreshing tokens, {self.token_refreshes} out of {self.MAX_REFRESHES} refreshes, "
                         f"from {url}.")
            response = requests.post(url, headers=headers, verify=self.verify_cert)
            logging.debug('Response from refreshtoken post:\n'
                          f'\turl: {url}\n'
                          f'\theaders: {headers}\n'
                          f'\tresponse: {response}')
            self.token_refreshes += 1
        else:
            self.token_refreshes = 0
            self.token_creation_time = datetime.datetime.now()  # Can't trust that your clock is in sync with FMC's.
            headers = {'Content-Type': 'application/json'}
            url = f'https://{self.__host}/{self.API_PLATFORM_VERSION}/auth/generatetoken'
            logging.info(f"Requesting new tokens from {url}.")
            response = requests.post(url, headers=headers,
                                     auth=requests.auth.HTTPBasicAuth(self.__username, self.__password),
                                     verify=self.verify_cert)
            logging.debug('Response from generatetoken post:\n'
                          f'\turl: {url}\n'
                          f'\theaders: {headers}\n'
                          f'\tresponse: {response}')
        self.access_token = response.headers.get('X-auth-access-token')
        self.refresh_token = response.headers.get('X-auth-refresh-token')
        self.uuid = response.headers.get('DOMAIN_UUID')
        all_domain = json.loads(response.headers.get('DOMAINS'))
        if self.__domain is not None:
            for domain in all_domain:
                if 'global/' + self.__domain.lower() == domain['name'].lower():
                    logging.info(f"Domain set to {domain['name']}")
                    self.uuid = domain['uuid']
                else:
                    logging.info("Domain name entered not found in FMC, falling back to Global")

    def get_token(self):
        """
        Check validity of current token.  If needed make a new or refresh.  Then return access_token.
        :return self.access_token
        """
        logging.debug("In the Token get_token() class method.")
        if datetime.datetime.now() > (self.token_creation_time + datetime.timedelta(seconds=self.TOKEN_REFRESH_TIME)):
            logging.info("Token expired.  Generating a new token.")
            self.token_refreshes = 0
            self.access_token = None
            self.refresh_token = None
            self.generate_tokens()

        return self.access_token



import logging


def test__auditrecords(fmc):
    logging.info('Testing fmc.auditrecords() method.')
    logging.info(fmc.auditrecords())
    logging.info('Testing fmc.auditrecords() method done.\n')



import logging


def test__deploymentrequests(fmc):
    logging.info('Testing fmc.deploymentrequests() method.')
    logging.info(fmc.deploymentrequests())
    logging.info('Testing fmc.deploymentrequests() method done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.device_services.devicerecords import Device
import logging
import warnings


class FTDDeviceHAPairs(APIClassTemplate):
    """
    The FTDDeviceHAPairs Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'primary', 'secondary', 'ftdHABootstrap', 'action', 'forceBreak']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/devicehapairs/ftddevicehapairs'
    REQUIRED_FOR_POST = ['primary', 'secondary', 'ftdHABootstrap']
    REQUIRED_FOR_PUT = ['id']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for FTDDeviceHAPairs class.")
        self.parse_kwargs(**kwargs)

    def device(self, primary_name="", secondary_name=""):
        logging.debug("In device() for FTDDeviceHAPairs class.")
        primary = Device(fmc=self.fmc)
        primary.get(name=primary_name)
        secondary = Device(fmc=self.fmc)
        secondary.get(name=secondary_name)
        if 'id' in primary.__dict__:
            self.primary_id = primary.id
        else:
            logging.warning(f'Device {primary_name} not found.  Cannot set up device for FTDDeviceHAPairs.')
        if 'id' in secondary.__dict__:
            self.secondary_id = secondary.id
        else:
            logging.warning(f'Device {secondary_name} not found.  Cannot set up device for FTDDeviceHAPairs.')

    def primary(self, name):
        logging.debug("In primary() for FTDDeviceHAPairs class.")
        primary = Device(fmc=self.fmc)
        primary.get(name=name)
        if 'id' in primary.__dict__:
            self.primary = {"id": primary.id}
        else:
            logging.warning(f'Device {primary.name} not found.  Cannot set up device for FTDDeviceHAPairs.')

    def secondary(self, name):
        logging.debug("In secondary() for DeviceHAPairs class.")
        secondary = Device(fmc=self.fmc)
        secondary.get(name=name)
        if 'id' in secondary.__dict__:
            self.secondary = {"id": secondary.id}
        else:
            logging.warning(f'Device {secondary.name} not found.  Cannot set up device for FTDDeviceHAPairs.')

    def switch_ha(self):
        logging.debug("In switch_ha() for FTDDeviceHAPairs class.")
        ha1 = DeviceHAPairs(fmc=self.fmc)
        ha1.get(name=self.name)
        if 'id' in ha1.__dict__:
            self.id = ha1.id
            self.action = "SWITCH"
        else:
            logging.warning(f'FTDDeviceHAPairs {self.name} not found.  Cannot set up HA for SWITCH.')

    def break_ha(self):
        logging.debug("In break_ha() for FTDDeviceHAPairs class.")
        ha1 = DeviceHAPairs(fmc=self.fmc)
        ha1.get(name=self.name)
        if 'id' in ha1.__dict__:
            self.id = ha1.id
            self.action = "HABREAK"
            self.forceBreak = True
        else:
            logging.warning(f'FTDDeviceHAPairs {self.name} not found.  Cannot set up HA for BREAK.')

    def post(self, **kwargs):
        logging.debug("In post() for FTDDeviceHAPairs class.")
        # Attempting to "Deploy" during Device registration causes issues.
        self.fmc.autodeploy = False
        return super().post(**kwargs)

    def put(self, **kwargs):
        logging.debug("In put() for FTDDeviceHAPairs class.")
        # Attempting to "Deploy" during Device registration causes issues.
        self.fmc.autodeploy = False
        return super().put(**kwargs)


class DeviceHAPairs(FTDDeviceHAPairs):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: DeviceHAPairs() should be called via FTDDeviceHAPairs().")
        super().__init__(fmc, **kwargs)



import logging
from .bridgegroupinterfaces import BridgeGroupInterfaces
from .devicerecords import Device
from .devicerecords import DeviceRecords
from .etherchannelinterfaces import EtherchannelInterfaces
from .ipv4staticroutes import IPv4StaticRoutes
from .ipv4staticroutes import IPv4StaticRoute
from .ipv6staticroutes import IPv6StaticRoutes
from .ipv6staticroutes import IPv6StaticRoute
from .physicalinterfaces import PhysicalInterfaces
from .physicalinterfaces import PhysicalInterface
from .redundantinterfaces import RedundantInterfaces
from .staticroutes import StaticRoutes
from .subinterfaces import SubInterfaces

logging.debug("In the device_services __init__.py file.")

__all__ = [
    'Device',
    'DeviceRecords',
    'StaticRoutes',
    'IPv4StaticRoutes',
    'IPv4StaticRoute',
    'IPv6StaticRoutes',
    'IPv6StaticRoute',
    'PhysicalInterfaces',
    'PhysicalInterface',
    'BridgeGroupInterfaces',
    'RedundantInterfaces',
    'EtherchannelInterfaces',
    'SubInterfaces'
]



import logging
import fmcapi
import time


def test__vlan_group_tag(fmc):
    logging.info('Testing VlanGroupTag class.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    obj10 = fmcapi.VlanTags(fmc=fmc, name='_vlantag10', data={'startTag': '888', 'endTag': '999'})
    obj10.post()
    obj11 = fmcapi.VlanTags(fmc=fmc, name='_vlantag11', data={'startTag': '222', 'endTag': '333'})
    obj11.post()
    obj12 = fmcapi.VlanTags(fmc=fmc, name='_vlantag12', data={'startTag': '1', 'endTag': '999'})
    obj12.post()
    time.sleep(1)
    obj1 = fmcapi.VlanGroupTags(fmc=fmc, name=namer)
    obj1.named_vlantags(action='add', name=obj10.name)
    obj1.named_vlantags(action='add', name=obj11.name)
    obj1.named_vlantags(action='remove', name=obj11.name)
    obj1.named_vlantags(action='clear')
    obj1.named_vlantags(action='add', name=obj10.name)
    obj1.named_vlantags(action='add', name=obj11.name)
    obj1.named_vlantags(action='add', name=obj12.name)
    obj1.named_vlantags(action='remove', name=obj12.name)
    obj1.post()
    time.sleep(1)
    del obj1
    obj1 = fmcapi.VlanGroupTags(fmc=fmc, name=namer)
    obj1.get()
    obj1.unnamed_vlantags(action='add', startvlan='22', endvlan='33')
    obj1.unnamed_vlantags(action='clear')
    obj1.unnamed_vlantags(action='add', startvlan='22', endvlan='33')
    obj1.unnamed_vlantags(action='remove', startvlan='22', endvlan='33')
    obj1.unnamed_vlantags(action='add', startvlan='44', endvlan='33')
    obj1.unnamed_vlantags(action='add', startvlan='900')
    obj1.put()
    time.sleep(1)
    obj1.delete()
    obj10.delete()
    obj11.delete()
    obj12.delete()

    logging.info('Testing VlanGroupTag class done.\n')



import logging
from .url_category import test__url_category
from .ports import test__ports
from .upgrades import test__upgrades
from .manualnat import test__manualnat
from .autonat import test__autonat
from .port_object_group import test__port_object_group
from .acprule import test__acp_rule
from .acp import test__access_control_policy
from .intrusion_policy import test__intrusion_policy
from .interfaces_subinterfaces import test__subinterfaces
from .interfaces_etherchannel import test__etherchannel_interfaces
from .interfaces_redundant import test__redundant_interfaces
from .interfaces_bridge_group import test__bridge_group_interfaces
from .interfaces_physical import test__phys_interfaces
from .wait_for_task import wait_for_task
from .device_with_task import test__device_with_task
from .sla_monitor import test__slamonitor
from .interface_group import test__interface_group
from .security_zone import test__security_zone
from .protocol_port import test__protocol_port
from .vlan_tag import test__vlan_tag
from .url import test__url
from .ikev2 import test__ikev2
from .ikev1 import test__ikev1
from .icmpv6 import test__icmpv6
from .icmpv4 import test__icmpv4
from .geolocations import test__geolocations
from .acls_extended import test__extended_acls
from .ip_range import test__ip_range
from .ip_network import test__ip_network
from .ip_host import test__ip_host
from .variable_set import test__variable_set
from .fmc_version import test__fmc_version
from .ip_addresses import test__ip_addresses
from .network_group import test__network_group
from .url_group import test__url_group
from .vlan_group_tag import test__vlan_group_tag
from .dns_servers_group import test__dns_servers_group
from .continent import test__continent
from .file_policies import test__filepolicies
from .country import test__country
from .certificate_enrollment import test__cert_enrollment
from .application_category import test__application_category
from .application_productivity import test__application_productivity
from .application_filter import test__application_filter
from .application_risk import test__application_risk
from .application import test__application
from .application_tag import test__application_tag
from .application_type import test__application_type

from .auditrecords import test__auditrecords
from .deployabledevices import test__deployabledevices
from .deploymentrequests import test__deploymentrequests
from .devicegrouprecords import test__devicegrouprecords
from .ftddevicehapairs import test__ftddevicehapairs
from .failoverinterfacemacaddressconfigs import test__failoverinterfacemacaddressconfigs
from .monitored_interface import test__monitoredinterfaces
from .devicerecords import test__devicerecords
from .staticroutes import test__staticroutes
from .ipv4staticroutes import test__ipv4staticroutes
from .ipv6staticroutes import test__ipv6staticroutes

logging.debug("In the unit-tests __init__.py file.")

__all__ = [
    'test__auditrecords',
    'test__deploymentrequests',
    'test__deployabledevices',
    'test__devicegrouprecords',
    'test__ftddevicehapairs',
    'test__failoverinterfacemacaddressconfigs',
    'test__monitoredinterfaces',
    'test__devicerecords',
    'test__staticroutes',
    'test__ipv4staticroutes',
    'test__ipv6staticroutes',
    'test__bridge_group_interfaces',

    'test__url_category',
    'test__application_type',
    'test__application_tag',
    'test__application',
    'test__application_risk',
    'test__application_filter',
    'test__application_productivity',
    'test__application_category',
    'test__cert_enrollment',
    'test__country',
    'test__filepolicies',
    'test__continent',
    'test__dns_servers_group',
    'test__vlan_group_tag',
    'test__url_group',
    'test__network_group',
    'test__ip_addresses',
    'test__fmc_version',
    'test__variable_set',
    'test__ip_host',
    'test__ip_network',
    'test__ip_range',
    'test__extended_acls',
    'test__geolocations',
    'test__icmpv6',
    'test__icmpv4',
    'test__ikev2',
    'test__ikev1',
    'test__vlan_tag',
    'test__url',
    'test__protocol_port',
    'test__security_zone',
    'test__interface_group',
    'test__slamonitor',
    'test__device_with_task',
    'wait_for_task',
    'test__phys_interfaces',
    'test__redundant_interfaces',
    'test__etherchannel_interfaces',
    'test__subinterfaces',
    'test__ports',
    'test__upgrades',
    'test__manualnat',
    'test__autonat',
    'test__port_object_group',
    'test__acp_rule',
    'test__access_control_policy',
    'test__intrusion_policy',
]



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .ftdnatpolicies import FTDNatPolicies
from fmcapi.api_objects.object_services.networkaddresses import NetworkAddresses
from fmcapi.api_objects.object_services.interfaceobjects import InterfaceObjects
from fmcapi.api_objects.object_services.networkgroups import NetworkGroups
import logging


class AutoNatRules(APIClassTemplate):
    """
    The AutoNatRules Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'originalNetwork', 'translatedNetwork', 'interfaceInTranslatedNetwork',
                       'natType', 'interfaceIpv6', 'fallThrough', 'dns', 'routeLookup', 'noProxyArp', 'netToNet',
                       'sourceInterface', 'destinationInterface', 'originalPort', 'translatedPort', 'serviceProtocol',
                       'patOptions',
                       ]
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    PREFIX_URL = '/policy/ftdnatpolicies'
    REQUIRED_FOR_POST = ["nat_id"]

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for AutoNatRules class.")
        self.parse_kwargs(**kwargs)
        self.type = "FTDAutoNatRule"

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for AutoNatRules class.")
        if 'translatedNetwork' in kwargs and 'interfaceInTranslatedNetwork' is True:
            logging.warning("Cannot have both a translatedNetwork and interfaceInTranslatedNetwork")
        elif 'translatedNetwork' in kwargs:
            self.translatedNetwork = kwargs['translatedNetwork']
        elif 'interfaceInTranslatedNetwork' in kwargs:
            self.interfaceInTranslatedNetwork = kwargs['interfaceInTranslatedNetwork']

    def nat_policy(self, name):
        logging.debug("In nat_policy() for AutoNatRules class.")
        ftd_nat = FTDNatPolicies(fmc=self.fmc)
        ftd_nat.get(name=name)
        if 'id' in ftd_nat.__dict__:
            self.nat_id = ftd_nat.id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.nat_id}/autonatrules'
            self.nat_added_to_url = True
        else:
            logging.warning(f'FTD NAT Policy {name} not found.  Cannot set up AutoNatRule for NAT Policy.')

    def original_network(self, name):
        logging.debug("In original_network() for AutoNatRules class.")
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to originalNetwork.')
        else:
            self.originalNetwork = new_net
            logging.info(f'Adding "{name}" to sourceNetworks for this AutoNatRule.')

    def translated_network(self, name):
        # Auto Nat rules can't use network group objects
        logging.debug("In translated_network() for AutoNatRules class.")
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to translatedNetwork.')
        else:
            self.translatedNetwork = new_net
            logging.info(f'Adding "{name}" to destinationNetworks for this AutoNatRule.')

    def source_intf(self, name):
        logging.debug("In source_intf() for AutoNatRules class.")
        intf_obj = InterfaceObjects(fmc=self.fmc).get()
        items = intf_obj.get('items', [])
        new_intf = None
        for item in items:
            if item["name"] == name:
                new_intf = {'id': item['id'], 'type': item['type']}
                break
        if new_intf is None:
            logging.warning(f'Interface Object "{name}" is not found in FMC.  Cannot add to sourceInterface.')
        else:
            if new_intf['type'] == "InterfaceGroup" and len(new_intf.interfaces) > 1:
                logging.warning(f'Interface Object "{name}" contains more than one physical interface. Cannot add to '
                                f'sourceInterface.')
            else:
                self.sourceInterface = new_intf
                logging.info(f'Interface Object "{name}" added to NAT Policy.')

    def destination_intf(self, name):
        logging.debug("In destination_intf() for AutoNatRules class.")
        intf_obj = InterfaceObjects(fmc=self.fmc).get()
        items = intf_obj.get('items', [])
        new_intf = None
        for item in items:
            if item["name"] == name:
                new_intf = {'id': item['id'], 'type': item['type']}
                break
        if new_intf is None:
            logging.warning(f'Interface Object "{name}" is not found in FMC.  Cannot add to destinationInterface.')
        else:
            if new_intf['type'] == "InterfaceGroup" and len(new_intf.interfaces) > 1:
                logging.warning(f'Interface Object "{name}" contains more than one physical interface. Cannot add to '
                                f'destinationInterface.')
            else:
                self.destinationInterface = new_intf
                logging.info(f'Interface Object "{name}" added to NAT Policy.')

    def identity_nat(self, name):
        logging.debug("In identity_nat() for AutoNatRules class.")
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to this AutoNatRule.')
        else:
            self.natType = "STATIC"
            self.originalNetwork = new_net
            self.translatedNetwork = new_net
            logging.info(f'Adding "{name}" to AutoNatRule.')

    def patPool(self, name, options={}):
        # Network Group Object permitted for patPool
        ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
        networkgroup_json = NetworkGroups(fmc=self.fmc).get()
        items = ipaddresses_json.get('items', []) + networkgroup_json.get('items', [])
        new_net = None
        for item in items:
            if item['name'] == name:
                new_net = {'name': item['name'], 'id': item['id'], 'type': item['type']}
                break
        if new_net is None:
            logging.warning(f'Network "{name}" is not found in FMC.  Cannot add to patPool.')
        else:
            self.natType = "DYNAMIC"
            self.patOptions = {"patPoolAddress": new_net}
            self.patOptions["interfacePat"] = options.interfacePat if "interfacePat" in options.keys() else False
            self.patOptions["includeReserve"] = options.includeReserve if "includeReserve" in options.keys() else False
            self.patOptions["roundRobin"] = options.roundRobin if "roundRobin" in options.keys() else True
            self.patOptions["extendedPat"] = options.extendedPat if "extendedPat" in options.keys() else False
            self.patOptions["flatPortRange"] = options.flatPortRange if "flatPortRange" in options.keys() else False
            logging.info(f'Adding "{name}" to patPool for this AutoNatRule.')



import logging
from .devicegrouprecords import DeviceGroupRecords
from .devicegrouprecords import DeviceGroups

logging.debug("In the device_group_services __init__.py file.")

__all__: [
    'DeviceGroupRecords',
    'DeviceGroups',
]



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging


class IKEv1Policies(APIClassTemplate):
    """
    The IKEv1Policies Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'encryption', 'hash', 'priority', 'diffieHellmanGroup',
                       'authenticationMethod', 'lifetimeInSeconds',
                       ]
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/ikev1policies'
    REQUIRED_FOR_POST = ['name', 'encryption', 'hash', 'diffieHellmanGroup', 'lifetimeInSeconds',
                         'authenticationMethod',
                         ]
    VALID_FOR_ENCRYPTION = ['DES', '3DES', 'AES-128', 'AES-192', 'AES-256']
    VALID_FOR_HASH = ['MD5', 'SHA']
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\- ]"""
    FIRST_SUPPORTED_FMC_VERSION = '6.3'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for IKEv1Policies class.")
        self.parse_kwargs(**kwargs)
        self.type = 'Ikev1PolicyObject'

    def format_data(self):
        json_data = super().format_data()
        logging.debug("In format_data() for IKEv1Policies class.")
        if 'encryption' in self.__dict__:
            if self.encryption in self.VALID_FOR_ENCRYPTION:
                json_data['encryption'] = self.encryption
            else:
                logging.warning(f'encryption "{self.encryption}" not a valid type.')
        if 'hash' in self.__dict__:
            if self.hash in self.VALID_FOR_HASH:
                json_data['hash'] = self.hash
            else:
                logging.warning(f'hash "{self.hash}" not a valid type.')
        return json_data



import logging
import fmcapi
import time
from unit_tests import wait_for_task


def test__device_with_task(fmc):
    logging.info('Test Device1 with Task.  This requires having an actual device with the "configure manager add" '
                 'statement enabled.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    acp1 = fmcapi.AccessPolicies(fmc=fmc, name=namer)
    acp1.post()

    starttime = str(int(time.time()))
    obj1_namer = f'_fmcapi_test_{starttime}'

    obj1 = fmcapi.Device(fmc=fmc)
    obj1.hostName = "10.255.0.43"
    obj1.name = obj1_namer
    obj1.regKey = "cisco123"
    obj1.natID = "cisco123"
    obj1.acp(name=acp1.name)
    obj1.licensing(action='add', name='BASE')
    obj1.licensing(action='add', name='THREAT')
    obj1.licensing(action='add', name='MALWARE')
    logging.info('Device -->')
    logging.info(obj1.format_data())

    response = obj1.post()
    wait_for_task(response["metadata"]["task"], 30)
    logging.info('Test Device2 with Task.  This requires having an actual device with the "configure manager add" '
                 'statement enabled.')

    starttime = str(int(time.time()))
    obj2_namer = f'_fmcapi_test_{starttime}'

    obj2 = fmcapi.Device(fmc=fmc)
    obj2.hostName = "10.255.0.44"
    obj2.name = obj2_namer
    obj2.regKey = "cisco123"
    obj2.natID = "cisco123"
    obj2.acp(name=acp1.name)
    obj2.licensing(action='add', name='BASE')
    obj2.licensing(action='add', name='THREAT')
    obj2.licensing(action='add', name='MALWARE')
    logging.info('Device -->')
    logging.info(obj2.format_data())
    logging.info('\n')
    obj2.post()
    # wait_for_task(response["metadata"]["task"], 30)

    # Wait some additional time to complete device registration before deletion
    time.sleep(180)
    obj1 = fmcapi.Device(fmc=fmc)
    obj2 = fmcapi.Device(fmc=fmc)
    obj1.get(name=obj1_namer)
    obj2.get(name=obj2_namer)

    obj1.delete()
    time.sleep(30)
    obj2.delete()
    time.sleep(30)
    acp1.delete()



from setuptools import setup, find_packages

__author__ = "Dax Mickelson"
__author_email = "dmickels@cisco.com"
__license__ = "BSD"

setup(
    name='fmcapi',
    version='20191002.0',
    description="Easier interface to Cisco's FMC API than writing your own way.",
    long_description="""With the removal to configure a Cisco NGFW via the command line your only option is to
     do so via a manager.  Some things are better when automated so using the manager's API gives us that power. 
     However, the current way to write external scripts and interact with the FMC's API isn't that great.  We created 
     this "wrapper" to give you an easier to use way of communicating with the FMC's API than using the example code 
     provided in the API Explorer.""",
    url='https://github.com/daxm/fmcapi',
    author='Dax Mickelson',
    author_email='dmickels@cisco.com',
    license='BSD',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Developers',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS',
        'Operating System :: Microsoft',
        'Programming Language :: Python :: 3',
        'Topic :: Security',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Networking :: Firewalls',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities',
    ],
    keywords='fmcapi fmc ftd security cisco ngfw api firepower',
    packages=find_packages(exclude=['docs', 'tests*']),
    install_requires=['requests', 'datetime', 'ipaddress'],
    python_requires='>=3',
    package_data={},
    data_files=None,
)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class ProtocolPortObjects(APIClassTemplate):
    """
    The ProtocolPortObjects in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'description', 'port', 'protocol', 'type']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/protocolportobjects'
    REQUIRED_FOR_POST = ['name', 'port', 'protocol']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for ProtocolPortObjects class.")
        self.parse_kwargs(**kwargs)


class ProtocolPort(ProtocolPortObjects):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: ProtocolPort() should be called via ProtocolPortObjects().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi
import time


def test__slamonitor(fmc):
    logging.info('Test SLAMonitor.  Post, get, put, delete SLAMonitor Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    sz1 = fmcapi.SecurityZones(fmc=fmc)
    sz1.name = "SZ-OUTSIDE1"
    sz1.interfaceMode = 'ROUTED'
    sz1.post()
    time.sleep(1)

    sz2 = fmcapi.SecurityZones(fmc=fmc)
    sz2.name = "SZ-OUTSIDE2"
    sz2.interfaceMode = 'ROUTED'
    sz2.post()
    time.sleep(1)

    obj1 = fmcapi.SLAMonitors(fmc=fmc)
    obj1.name = namer
    obj1.frequency = 30
    obj1.slaId = 1
    obj1.monitorAddress = "8.8.8.7"
    obj1.timeout = 5000
    obj1.threshold = 2
    obj1.noOfPackets = 1
    obj1.dataSize = 28
    obj1.tos = 1
    obj1.interfaces(names=["SZ-OUTSIDE1", "SZ-OUTSIDE2"])
    obj1.post()
    logging.info('SLAMonitor Post -->')
    logging.info(obj1.format_data())
    logging.info('\n')
    obj1.get(name=namer)
    obj1.monitorAddress = "8.8.8.8"
    obj1.put()
    logging.info('SLAMonitor Put -->')
    logging.info(obj1.format_data())
    logging.info('\n')
    time.sleep(1)
    obj1.delete()
    sz1.delete()
    sz2.delete()



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .devicerecords import DeviceRecords
from fmcapi.api_objects.object_services.networkaddresses import NetworkAddresses
from fmcapi.api_objects.object_services.hosts import Hosts
from fmcapi.api_objects.object_services.networkgroups import NetworkGroups
import logging
import warnings


class IPv6StaticRoutes(APIClassTemplate):
    """
    The IPv6StaticRoute Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'interfaceName', 'selectedNetworks', 'gateway', 'routeTracking', 'metricValue',
                       'isTunneled',
                       ]
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['device_name']
    PREFIX_URL = '/devices/devicerecords'
    URL_SUFFIX = None
    REQUIRED_FOR_POST = ['interfaceName', 'selectedNetworks', 'gateway']
    REQUIRED_FOR_PUT = ['id', 'device_id']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for IPv6StaticRoute class.")
        self.type = 'IPv6StaticRoute'
        self.parse_kwargs(**kwargs)

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for IPv6StaticRoute class.")
        if 'device_name' in kwargs:
            self.device(device_name=kwargs['device_name'])

    def device(self, device_name):
        logging.debug("In device() for IPv6StaticRoute class.")
        device1 = DeviceRecords(fmc=self.fmc)
        device1.get(name=device_name)
        if 'id' in device1.__dict__:
            self.device_id = device1.id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.device_id}/routing/ipv6staticroutes'
            self.device_added_to_url = True
        else:
            logging.warning(f'Device {device_name} not found.  Cannot set up device for IPv6StaticRoute.')

    def networks(self, action, networks):
        logging.info("In networks() for IPv6StaticRoute class.")
        if action == 'add':
            # Valid objects are IPHost, IPNetwork and NetworkGroup.
            # Create a dictionary to contain all three object type.
            ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
            networkgroup_json = NetworkGroups(fmc=self.fmc).get()
            items = ipaddresses_json.get('items', []) + networkgroup_json.get('items', [])
            for network in networks:
                # Find the matching object name in the dictionary if it exists
                net1 = list(filter(lambda i: i['name'] == network, items))
                if len(net1) > 0:
                    if 'selectedNetworks' in self.__dict__:
                        # Check to see if network already exists
                        exists = list(filter(lambda i: i['id'] == net1[0]['id'], self.selectedNetworks))
                        if 'id' in exists:
                            logging.warning(f'Network "{network}" already exists in selectedNetworks.')
                        else:
                            self.selectedNetworks.append(
                                {"type": net1[0]['type'], "id": net1[0]['id'], "name": net1[0]['name']})
                    else:
                        self.selectedNetworks = [
                            {"type": net1[0]['type'], "id": net1[0]['id'], "name": net1[0]['name']}]
                else:
                    logging.warning(f'Network "{network}" not found.  Cannot set up device for IPv6StaticRoute.')
        elif action == 'remove':
            ipaddresses_json = NetworkAddresses(fmc=self.fmc).get()
            networkgroup_json = NetworkGroups(fmc=self.fmc).get()
            items = ipaddresses_json.get('items', []) + networkgroup_json.get('items', [])
            for network in networks:
                net1 = list(filter(lambda i: i['name'] == network, items))
                if len(net1) > 0:
                    if 'selectedNetworks' in self.__dict__:
                        new_net1 = list(filter(lambda i: i['id'] != net1[0]['id'], self.selectedNetworks))
                    else:
                        logging.warning("No selectedNetworks found for this Device's IPv6StaticRoute.")
                else:
                    logging.warning(f'Network "{network}" not found.  Cannot set up device for IPv6StaticRoute.')
        elif action == 'clear':
            if 'selectedNetworks' in self.__dict__:
                del self.selectedNetworks
                logging.info('All selectedNetworks removed from this IPv6StaticRoute object.')

    def gw(self, name):
        logging.info("In gw() for IPv6StaticRoute class.")
        gw1 = Hosts(fmc=self.fmc)
        gw1.get(name=name)
        if 'id' in gw1.__dict__:
            self.gateway = {
                "object": {
                    "type": gw1.type,
                    "id": gw1.id,
                    "name": gw1.name}}
        else:
            logging.warning(f'Network "{name}" not found.  Cannot set up device for IPv6StaticRoute.')


class IPv6StaticRoute(IPv6StaticRoutes):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: IPv6StaticRoute() should be called via IPv6StaticRoutes().")
        super().__init__(fmc, **kwargs)



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class VariableSets(APIClassTemplate):
    """
    The VariableSets Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type', 'description']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    URL_SUFFIX = '/object/variablesets'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for VariableSets class.")
        self.parse_kwargs(**kwargs)

    def post(self):
        logging.info('POST method for API for VariableSets not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for VariableSets not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for VariableSets not supported.')
        pass


class VariableSet(VariableSets):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: VariableSet() should be called via VariableSets().")
        super().__init__(fmc, **kwargs)



import logging
from .accesspolicies import AccessPolicies
from .accesspolicies import AccessControlPolicy
from .accessrules import AccessRules
from .accessrules import ACPRule
from .accessrules import Bulk
from .autonatrules import AutoNatRules
from .filepolicies import FilePolicies
from .ftdnatpolicies import FTDNatPolicies
from .ftdnatpolicies import FTDNatPolicy
from .hitcounts import HitCounts
from .hitcounts import HitCount
from .intrusionpolicies import IntrusionPolicies
from .intrusionpolicies import IntrusionPolicy
from .manualnatrules import ManualNatRules
from .natrules import NatRules
from .prefilterpolicies import PreFilterPolicies
from .prefilterpolicies import PreFilterPolicy
from .defaultactions import DefaultActions

logging.debug("In the object_services __init__.py file.")

__all__ = [
    'AccessPolicies',
    'AccessControlPolicy',
    'AccessRules',
    'ACPRule',
    'Bulk',
    'FilePolicies',
    'FTDNatPolicies',
    'FTDNatPolicy',
    'AutoNatRules',
    'ManualNatRules',
    'NatRules',
    'IntrusionPolicies',
    'IntrusionPolicy',
    'PreFilterPolicies',
    'PreFilterPolicy',
    'HitCounts',
    'HitCount',
    'DefaultActions'
]



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .ftddevicehapairs import FTDDeviceHAPairs
from fmcapi.api_objects.device_services.physicalinterfaces import PhysicalInterfaces
import logging
import warnings


class FailoverInterfaceMACAddressConfigs(APIClassTemplate):
    """
    The FailoverInterfaceMACAddressConfigs Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'physicalInterface', 'failoverActiveMac','failoverStandbyMac']
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['ha_name']
    PREFIX_URL = '/devicehapairs/ftddevicehapairs'
    REQUIRED_FOR_POST = ['physicalInterface', 'failoverActiveMac', 'failoverStandbyMac']
    REQUIRED_FOR_PUT = ['id']

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for FailoverInterfaceMACAddressConfigs class.")
        self.parse_kwargs(**kwargs)

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for FailoverInterfaceMACAddressConfigs class.")
        if 'ha_name' in kwargs:
            self.device_ha(ha_name=kwargs['ha_name'])

    def device_ha(self, ha_name):
        logging.debug("In device_ha() for FailoverInterfaceMACAddressConfigs class.")
        deviceha1 = FTDDeviceHAPairs(fmc=self.fmc, name=ha_name)
        deviceha1.get()
        if 'id' in deviceha1.__dict__:
            self.deviceha_id = deviceha1.id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/' \
                       f'{self.deviceha_id}/failoverinterfacemacaddressconfigs'
            self.deviceha_added_to_url = True
        else:
            logging.warning(f'Device HA {ha_name} not found. '
                            f'Cannot set up device for FailoverInterfaceMACAddressConfigs.')

    def p_interface(self, name, device_name):
        logging.debug("In p_interface() for FailoverInterfaceMACAddressConfigs class.")
        intf1 = PhysicalInterfaces(fmc=self.fmc)
        intf1.get(name=name, device_name=device_name)
        if 'id' in intf1.__dict__:
            self.physicalInterface = {'name': intf1.name, 'id': intf1.id, 'type': intf1.type}
        else:
            logging.warning(f'PhysicalInterface, "{name}", not found.  '
                            f'Cannot add to FailoverInterfaceMACAddressConfigs.')

    def edit(self, name, ha_name):
        logging.debug("In edit() for FailoverInterfaceMACAddressConfigs class.")
        deviceha1 = FTDDeviceHAPairs(fmc=self.fmc, name=ha_name)
        deviceha1.get()
        obj1 = FailoverInterfaceMACAddressConfigs(fmc=self.fmc)
        obj1.device_ha(ha_name=ha_name)
        failovermac_json = obj1.get()
        items = failovermac_json.get('items', [])
        found = False
        for item in items:
            if item['physicalInterface']['name'] == name:
                found = True
                self.id = item['id']
                self.name = item['physicalInterface']['name']
                self.failoverActiveMac = item['failoverActiveMac']
                self.failoverStandbyMac = item['failoverStandbyMac']
                self.deviceha_id = deviceha1.id
                self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/' \
                           f'{self.deviceha_id}/failoverinterfacemacaddressconfigs'
                break
        if found is False:
            logging.warning(f'PhysicalInterface, "{name}", not found.  Cannot add to '
                            f'FailoverInterfaceMACAddressConfigs.')


class DeviceHAFailoverMAC(FailoverInterfaceMACAddressConfigs):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: DeviceHAFailoverMAC() should be called via FailoverInterfaceMACAddressConfigs().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi


def test__variable_set(fmc):
    logging.info('Test VariableSet. Can only GET VariableSet objects.')

    obj1 = fmcapi.VariableSets(fmc=fmc)
    obj1.get(name='Default-Set')
    logging.info('VariableSet -->')
    logging.info(obj1.format_data())

    logging.info('Test VariableSet done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from fmcapi.api_objects.policy_services.accesspolicies import AccessPolicies
from fmcapi.api_objects.device_services.devicerecords import DeviceRecords
from fmcapi.api_objects.device_ha_pair_services.ftddevicehapairs import FTDDeviceHAPairs
from fmcapi.api_objects.policy_services.ftdnatpolicies import FTDNatPolicies
import logging


class PolicyAssignments(APIClassTemplate):
    """
    The PolicyAssignments Object in the FMC.
    """
    VALID_JSON_DATA = ['id', 'name', 'type', 'targets', 'policy']
    VALID_FOR_KWARGS = VALID_JSON_DATA + []
    REQUIRED_FOR_POST = ['targets', 'policy']
    REQUIRED_FOR_PUT = ['id', 'targets', 'policy']
    URL_SUFFIX = '/assignment/policyassignments'
    FILTER_BY_NAME = True

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for PolicyAssignments class.")
        self.parse_kwargs(**kwargs)
        self.type = "PolicyAssignment"

    def ftd_natpolicy(self, name, devices):
        logging.debug("In ftd_natpolicy() for PolicyAssignments class.")
        targets = []
        pol1 = FTDNatPolicies(fmc=self.fmc)
        pol1.get(name=name)
        if 'id' in pol1.__dict__:
            self.policy = {"type": pol1.type, "name": pol1.name, "id": pol1.id}
        else:
            logging.warning(f'FTD NAT Policy {name} not found.  Cannot set up PolicyAssignment.')
        for device in devices:
            if device["type"] == 'device':
                dev1 = DeviceRecords(fmc=self.fmc)
                dev1.get(name=device['name'])
            elif device["type"] == 'deviceHAPair':
                dev1 = FTDDeviceHAPairs(fmc=self.fmc)
                dev1.get(name=device['name'])
            if 'id' in dev1.__dict__:
                logging.info(f'Adding "{dev1.name}" to targets for this FTDNat PolicyAssignment.')
                targets.append({"type": dev1.type, "id": dev1.id, "name": dev1.name})
            else:
                logging.warning(f"Device/DeviceHA {device['name']} not found.  Cannot add to PolicyAssignment.")
        self.targets = targets

    def accesspolicy(self, name, devices):
        logging.debug("In accesspolicy() for PolicyAssignments class.")
        targets = []
        pol1 = AccessPolicies(fmc=self.fmc)
        pol1.get(name=name)
        if 'id' in pol1.__dict__:
            self.policy = {"type": pol1.type, "name": pol1.name, "id": pol1.id}
        else:
            logging.warning(f'Access Control Policy {name} not found.  Cannot set up PolicyAssignment.')
        for device in devices:
            if device["type"] == 'device':
                dev1 = DeviceRecords(fmc=self.fmc)
                dev1.get(name=device['name'])
            elif device["type"] == 'deviceHAPair':
                dev1 = FTDDeviceHAPairs(fmc=self.fmc)
                dev1.get(name=device['name'])
            if 'id' in dev1.__dict__:
                logging.info(f'Adding "{dev1.name}" to targets for this Access Control Policy PolicyAssignment.')
                targets.append({"type": dev1.type, "id": dev1.id, "name": dev1.name})
            else:
                logging.warning(f"Device/DeviceHA {device['name']} not found.  Cannot add to PolicyAssignment.")
        self.targets = targets

    def delete(self):
        logging.info('DELETE method for API for PolicyAssignments not supported.')
        pass



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging
import warnings


class Applications(APIClassTemplate):
    """
    The Applications Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name', 'type']
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['appProductivity', 'appCategories', 'appTags', 'appId', 'risk',
                                          'applicationTypes',
                                          ]
    URL_SUFFIX = '/object/applications'
    VALID_CHARACTERS_FOR_NAME = """[.\w\d_\-\/\. ]"""

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for Applications class.")
        self.parse_kwargs(**kwargs)

    def post(self):
        logging.info('POST method for API for Applications not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for Applications not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for Applications not supported.')
        pass


class Application(Applications):
    """Dispose of this Class after 20210101."""

    def __init__(self, fmc, **kwargs):
        warnings.resetwarnings()
        warnings.warn("Deprecated: Application() should be called via Applications().")
        super().__init__(fmc, **kwargs)



import logging
import fmcapi
import time


def test__url_group(fmc):
    logging.info('Testing URLGroup class.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    url1 = fmcapi.URLs(fmc=fmc, name='_url1', url='example.org')
    url1.post()
    url2 = fmcapi.URLs(fmc=fmc, name='_url2', url='example.net')
    url2.post()
    url3 = fmcapi.URLs(fmc=fmc, name='_url3', url='example.com')
    url3.post()
    time.sleep(1)
    obj1 = fmcapi.URLGroups(fmc=fmc, name=namer)
    obj1.named_urls(action='add', name=url1.name)
    obj1.named_urls(action='add', name=url1.name)
    obj1.named_urls(action='clear')
    obj1.named_urls(action='add', name=url2.name)
    obj1.named_urls(action='add', name=url3.name)
    obj1.named_urls(action='add', name=url1.name)
    obj1.named_urls(action='remove', name=url3.name)
    obj1.post()
    time.sleep(1)
    del obj1
    obj1 = fmcapi.URLGroups(fmc=fmc, name=namer)
    obj1.get()
    obj1.unnamed_urls(action='add', value='daxm.net')
    obj1.unnamed_urls(action='add', value='daxm.com')
    obj1.unnamed_urls(action='clear')
    obj1.unnamed_urls(action='add', value='daxm.org')
    obj1.unnamed_urls(action='add', value='daxm.net')
    obj1.unnamed_urls(action='add', value='daxm.lan')
    obj1.unnamed_urls(action='remove', value='daxm.org')
    obj1.put()
    time.sleep(1)
    obj1.delete()
    url1.delete()
    url2.delete()
    url3.delete()

    logging.info('Testing URLGroup class done.\n')



import logging
import fmcapi
import time


def test__redundant_interfaces(fmc):
    logging.info(
        'Test RedundantInterfaces.  get, post, put, delete RedundantInterfaces Objects. Requires registered device')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    sz1 = fmcapi.SecurityZones(fmc=fmc)
    sz1.name = "_sz1" + namer
    sz1.post()
    time.sleep(1)
    sz2 = fmcapi.SecurityZones(fmc=fmc)
    sz2.name = "_sz2" + namer
    sz2.post()
    time.sleep(1)

    red1 = fmcapi.RedundantInterfaces(fmc=fmc, device_name="device_name")
    red1.primary(p_interface="GigabitEthernet0/3", device_name="device_name")
    red1.secondary(p_interface="GigabitEthernet0/5", device_name="device_name")
    red1.enabled = "True"
    red1.ifname = "_red1" + namer
    red1.redundantId = "1"
    red1.static(ipv4addr="192.0.2.1", ipv4mask=24)
    red1.sz(name=sz1.name)
    red1.post()
    time.sleep(2)

    red1.get()
    logging.info(red1.format_data())
    red1.enabled = False
    red1.sz(name=sz2.name)
    red1.put()
    time.sleep(1)

    red1.get()
    logging.info(red1.format_data())
    red1.delete()
    sz1.delete()
    sz2.delete()

    logging.info('Testing RedundantInterfaces class done.\n')



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
import logging


class FTDDeviceCluster(APIClassTemplate):
    """
    The FTDDeviceCluster Object in the FMC.
    """

    VALID_JSON_DATA = ['id', 'name']
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['slavedevices', 'modelType', 'version', 'sw_version', 'healthStatus',
                                          'healthPolicy', 'model', 'modelNumber', 'accessPolicy',
                                          ]
    URL_SUFFIX = '/deviceclusters/ftddevicecluster'

    def __init__(self, fmc, **kwargs):
        super().__init__(fmc, **kwargs)
        logging.debug("In __init__() for FTDDeviceCluster class.")
        self.parse_kwargs(**kwargs)
        self.type = 'DeviceCluster'

    def post(self):
        logging.info('POST method for API for FTDDeviceCluster not supported.')
        pass

    def put(self):
        logging.info('PUT method for API for FTDDeviceCluster not supported.')
        pass

    def delete(self):
        logging.info('DELETE method for API for FTDDeviceCluster not supported.')
        pass



from fmcapi.api_objects.apiclasstemplate import APIClassTemplate
from .accesspolicies import AccessPolicies
import logging


class DefaultActions(APIClassTemplate):
    """
    The DefaultActions Object in the FMC.
    """

    VALID_JSON_DATA = []
    VALID_FOR_KWARGS = VALID_JSON_DATA + ['acp_id', 'acp_name', 'device_id', 'device_name', 'fetchZeroHitCount',
                                          'limit', 'action',
                                          ]
    PREFIX_URL = '/policy/accesspolicies'
    REQUIRED_FOR_PUT = ['acp_id', 'id', 'action']
    REQUIRED_FOR_GET = ['acp_id']
    VALID_ACTION = ['BLOCK', 'TRUST', 'PERMIT', 'NETWORK_DISCOVERY']

    def __init__(self, fmc, **kwargs):
        logging.debug("In __init__() for DefaultActions class.")
        super().__init__(fmc, **kwargs)
        self.parse_kwargs(**kwargs)
        self.type = 'AccessPolicyDefaultAction'
        self.URL = f'{self.URL}{self.URL_SUFFIX}'

    def format_data(self):
        json_data = super().format_data()
        logging.debug("In format_data() for DefaultActions class.")
        if 'action' in self.__dict__:
            if self.action in self.VALID_ACTION:
                json_data['action'] = self.action
            else:
                logging.warning(f'action variable must be one of: {self.VALID_ACTION}.')
        return json_data

    def parse_kwargs(self, **kwargs):
        super().parse_kwargs(**kwargs)
        logging.debug("In parse_kwargs() for DefaultActions class.")
        if 'acp_id' in kwargs:
            self.acp(acp_id=kwargs['acp_id'])
        if 'acp_name' in kwargs:
            self.acp(name=kwargs['acp_name'])
        if 'device_id' in kwargs:
            self.device(id=kwargs['device_id'])
        if 'device_name' in kwargs:
            self.device(name=kwargs['device_name'])
        if 'action' in kwargs:
            if kwargs['action'] in self.VALID_ACTION:
                self.action = kwargs['action']
            else:
                logging.warning(f'action variable must be one of: {self.VALID_ACTION}.')
            self.action = kwargs['action']

    def acp(self, name='', acp_id=''):
        # either name or id of the ACP should be given
        logging.debug("In acp() for DefaultActions class.")
        if acp_id != '':
            self.acp_id = acp_id
            self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.acp_id}'
            self.acp_added_to_url = True
        elif name != '':
            acp1 = AccessPolicies(fmc=self.fmc)
            acp1.get(name=name)
            if 'id' in acp1.__dict__:
                self.acp_id = acp1.id
                self.URL = f'{self.fmc.configuration_url}{self.PREFIX_URL}/{self.acp_id}/defaultactions'
                self.acp_added_to_url = True
            else:
                logging.warning(f'Access Control Policy "{name}" not found.  Cannot configure acp for DefaultActions.')
        else:
            logging.error('No accessPolicy name or id was provided.')

    def delete(self, **kwargs):
        logging.info('API DELETE method for DefaultActions not supported.')
        pass

    def post(self):
        logging.info('API POST method for DefaultActions not supported.')
        pass



"""
The fmcapi __init__.py file is called whenever someone imports the package into their program.
"""
import logging
from .fmc import FMC
from .api_objects import *

logging.debug("In the fmcapi __init__.py file.")


def __authorship__():
    """In the FMC __authorship__() class method:
***********************************************************************************************************************
This python module was created by Dax Mickelson along with LOTs of help from Ryan Malloy and Neil Patel.  Thank you to
the github community members who have also pitched in, especially Mark Sullivan and his team.
Feel free to send me comments/suggestions/improvements.  Either by email: dmickels@cisco.com or more importantly
via a Pull request from the github repository: https://github.com/daxm/fmcapi.
***********************************************************************************************************************
        """
    logging.debug(__authorship__.__doc__)


__authorship__()



import logging
import fmcapi
import time


def test__protocol_port(fmc):
    logging.info('Test ProtocolPort.  Post, get, put, delete Port Objects.')

    starttime = str(int(time.time()))
    namer = f'_fmcapi_test_{starttime}'

    obj1 = fmcapi.ProtocolPortObjects(fmc=fmc)
    obj1.name = namer
    obj1.port = '1234'
    obj1.protocol = 'TCP'
    obj1.post()
    time.sleep(1)
    del obj1
    obj1 = fmcapi.ProtocolPortObjects(fmc=fmc, name=namer)
    obj1.get()
    obj1.port = '5678'
    obj1.put()
    time.sleep(1)
    obj1.delete()
    logging.info('Test ProtocolPort done.\n')
